<?php declare(strict_types=1);

namespace ConnectPayPayment\Command;

use ConnectPayPayment\Components\Helper\OrderTransaction\OrderTransactionHelperInterface;
use ConnectPayPayment\Components\PaymentHandler\AbstractPaymentHandler;
use Shopware\Core\Framework\Adapter\Console\ShopwareStyle;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

class GetPaymentDetailsCommand extends Command
{
    private const TABLE_HEADER = [
        'Customer',
        'Email',
        'Bank',
        'Sum',
        'Currency',
        'Is terminal',
        'Payment status',
        'Payment details',
    ];
    protected static $defaultName = 'cp:payment:get:status';
    protected static $defaultDescription = 'Retrieve (update) payments status';
    private AbstractPaymentHandler $paymentHandler;
    private OrderTransactionHelperInterface $orderTransactionHelper;

    /**
     * @param AbstractPaymentHandler $paymentHandler
     * @param OrderTransactionHelperInterface $orderTransactionHelper
     */
    public function __construct(
        AbstractPaymentHandler $paymentHandler,
        OrderTransactionHelperInterface $orderTransactionHelper
    ) {
        $this->paymentHandler = $paymentHandler;
        $this->orderTransactionHelper = $orderTransactionHelper;

        parent::__construct();
    }

    /**
     * {@inheritDoc}
     */
    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $pluginData = [];
        foreach ($this->orderTransactionHelper->getConnectPayTransactions() as $orderTransaction) {
            if (!isset($orderTransaction->getCustomFields()['paymentId'])) {
                continue;
            }

            $response = $this->paymentHandler->checkPaymentStatus($orderTransaction);

            if (!$response) {
                $output->writeln('<error>Something went wrong, please, check logs</error>');

                return Command::FAILURE;
            }

            $responseDetails = $response->getResponseDetails();
            $pluginData[] = [
                "{$responseDetails['consumer']['firstName']} {$responseDetails['consumer']['lastName']}",
                $responseDetails['consumer']['email'],
                $responseDetails['payment']['providerName'] ?? '',
                $responseDetails['payment']['instructedAmount']['amount'],
                $responseDetails['payment']['instructedAmount']['currency'],
                $response->getIsTerminal() ? '<info>Yes</info>' : '<error>No</error>',
                $response->getCode(),
                $responseDetails['status']['details']['errorMessage'] ?? $responseDetails['status']['details']['providerStatusCode'],
            ];
        }

        if (!empty($pluginData)) {
            $io = new ShopwareStyle($input, $output);
            $io->title('ConnectPay payment details');
            $io->table(self::TABLE_HEADER, $pluginData);
        } else {
            $output->writeln('<comment>No payments to process</comment>');
        }

        return Command::SUCCESS;
    }
}
