<?php declare(strict_types=1);

namespace ConnectPayPayment\Command;

use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use ConnectPayPayment\Components\Service\ProvidersService\ProvidersServiceInterface;

class GetProvidersCommand extends Command
{
    protected static $defaultName = 'cp:providers:get';
    protected static $defaultDescription = 'Retrieve list of providers (banks)';

    private ProvidersServiceInterface $providersService;

    /**
     * @param ProvidersServiceInterface $providersService
     */
    public function __construct(
        ProvidersServiceInterface $providersService
    ) {
        $this->providersService = $providersService;

        parent::__construct();
    }

    /**
     * {@inheritDoc}
     */
    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        if ($this->providersService->retrieveProviders()) {
            $output->writeln('<info>Bank providers successfully received!</info>');

            return Command::SUCCESS;
        }
        $output->writeln('<error>Something went wrong, please, check logs</error>');

        return Command::FAILURE;
    }
}
