<?php declare(strict_types=1);

namespace ConnectPayPayment\Components\Client\Hydrator\Response\GetPaymentDetails;

use ConnectPayPayment\Components\Client\Response\GenericResponse;
use ConnectPayPayment\Components\Client\Response\GetPaymentDetailsResponse;
use ConnectPayPayment\Components\Helper\OrderTransaction\OrderTransactionHelperInterface;
use Shopware\Core\Framework\Uuid\Uuid;
use Shopware\Core\System\SalesChannel\Context\AbstractSalesChannelContextFactory;
use Shopware\Core\System\SalesChannel\SalesChannelContext;

class GetPaymentDetailsResponseHydrator implements GetPaymentDetailsResponseHydratorInterface
{
    private OrderTransactionHelperInterface $orderTransactionHelper;
    private AbstractSalesChannelContextFactory $salesChannelContextFactory;

    /**
     * @param OrderTransactionHelperInterface $orderTransactionHelper
     * @param AbstractSalesChannelContextFactory $salesChannelContextFactory
     */
    public function __construct(
        OrderTransactionHelperInterface $orderTransactionHelper,
        AbstractSalesChannelContextFactory $salesChannelContextFactory
    ) {
        $this->orderTransactionHelper = $orderTransactionHelper;
        $this->salesChannelContextFactory = $salesChannelContextFactory;
    }

    /**
     * {@inheritdoc}
     */
    public function hydrate(GenericResponse $genericResponse): GetPaymentDetailsResponse
    {
        $genericResponse = $genericResponse->getResponse();

        // retrieve data only for final result
        if ($genericResponse['status']['isTerminal']) {
            $salesChannelContext = $this->getSalesChannelContext(
                $genericResponse['payment']['remittanceInformationUnstructured']
            );
            $transaction = $this->orderTransactionHelper->getOrderTransaction(
                $genericResponse['identifiers']['merchantReference'],
                $salesChannelContext->getContext()
            );
        }

        $response = new GetPaymentDetailsResponse();
        $response->assign([
            'isTerminal' => $genericResponse['status']['isTerminal'],
            'code' => $genericResponse['status']['code'],
            'transaction' => $transaction ?? null,
            'salesChannelContext' => $salesChannelContext ?? null,
            'responseDetails' => $genericResponse,
        ]);

        return $response;
    }

    /**
     * Get sales channel context
     *
     * @param string $salesChannelId
     * @return SalesChannelContext|null
     */
    private function getSalesChannelContext(string $salesChannelId): ?SalesChannelContext
    {
        return $this->salesChannelContextFactory->create(Uuid::randomHex(), $salesChannelId);
    }
}
