<?php declare(strict_types=1);

namespace ConnectPayPayment\Components\Controller\Storefront;

use ConnectPayPayment\Components\Helper\OrderTransaction\OrderTransactionHelperInterface;
use ConnectPayPayment\Components\PaymentHandler\AbstractPaymentHandler;
use Psr\Log\LoggerInterface;
use Shopware\Core\Framework\Routing\Annotation\RouteScope;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Shopware\Storefront\Controller\StorefrontController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Annotation\Route;

/**
 * @RouteScope(scopes={"storefront"})
 */
class CallbackController extends StorefrontController
{
    private AbstractPaymentHandler $paymentHandler;
    private OrderTransactionHelperInterface $orderTransactionHelper;
    private LoggerInterface $logger;

    /**
     * @param AbstractPaymentHandler $paymentHandler
     * @param OrderTransactionHelperInterface $orderTransactionHelper
     * @param LoggerInterface $logger
     */
    public function __construct(
        AbstractPaymentHandler $paymentHandler,
        OrderTransactionHelperInterface $orderTransactionHelper,
        LoggerInterface $logger
    ) {
        $this->paymentHandler = $paymentHandler;
        $this->orderTransactionHelper = $orderTransactionHelper;
        $this->logger = $logger;
    }

    /**
     * Callback handler
     *
     * @Route("/payments/connect-pay/callback", defaults={"csrf_protected": false}, name="frontend.connect_pay.callback", methods={"POST"})
     * @param Request $request
     * @param SalesChannelContext $context
     * @return JsonResponse
     */
    public function callback(Request $request, SalesChannelContext $context): JsonResponse
    {
        $this->logger->debug('ConnectPay callback triggered');

        $identifiers = $request->request->get('identifiers');

        if ($identifiers && $identifiers['merchantReference']) {
            $orderTransaction = $this->orderTransactionHelper->getOrderTransaction(
                $identifiers['merchantReference'],
                $context->getContext()
            );

            if (!$orderTransaction) {
                $this->logger->error('Missing orderTransaction!');
            }

            $this->paymentHandler->checkPaymentStatus($orderTransaction);
        }

        return new JsonResponse();
    }
}
