<?php declare(strict_types=1);

namespace ConnectPayPayment\Components\Helper\OrderTransaction;

use ConnectPayPayment\ConnectPayPayment;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionEntity;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStates;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityCollection;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsAnyFilter;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\MultiFilter;
use Shopware\Core\System\SalesChannel\SalesChannelContext;

class OrderTransactionHelper implements OrderTransactionHelperInterface
{
    private EntityRepositoryInterface $orderTransactionRepository;

    public function __construct(EntityRepositoryInterface $orderTransactionRepository)
    {
        $this->orderTransactionRepository = $orderTransactionRepository;
    }

    /**
     * {@inheritDoc}
     */
    public function getConnectPayTransactions(): EntityCollection
    {
        $criteria = new Criteria();

        $criteria->addFilter(
            new MultiFilter(MultiFilter::CONNECTION_AND, [
                new EqualsFilter('paymentMethod.handlerIdentifier', ConnectPayPayment::PAYMENT_HANDLER_IDENTIFIER),
                new EqualsAnyFilter(
                    'stateMachineState.technicalName',
                    [OrderTransactionStates::STATE_OPEN, OrderTransactionStates::STATE_IN_PROGRESS]
                ),
            ])
        );

        return $this->orderTransactionRepository->search($criteria, Context::createDefaultContext())->getEntities();
    }

    /**
     * {@inheritDoc}
     */
    public function getOrderTransaction($transactionId, Context $context): ?OrderTransactionEntity
    {
        $criteria = new Criteria([$transactionId]);
        $criteria->addAssociation('order');

        return $this->orderTransactionRepository->search($criteria, $context)->first();
    }

    /**
     * {@inheritDoc}
     */
    public function savePaymentId(
        string $paymentId,
        OrderTransactionEntity $orderTransaction,
        SalesChannelContext $salesChannelContext
    ): void {
        $this->orderTransactionRepository->update([
            [
                'id' => $orderTransaction->getId(),
                'customFields' => [
                    'paymentId' => $paymentId,
                ],
            ],
        ], $salesChannelContext->getContext());
    }
}
