<?php declare(strict_types=1);

namespace ConnectPayPayment;

use ConnectPayPayment\Installer\PaymentInstaller;
use Shopware\Core\Content\Media\File\FileSaver;
use Shopware\Core\Framework\Plugin;
use Shopware\Core\Framework\Plugin\Context\ActivateContext;
use Shopware\Core\Framework\Plugin\Context\DeactivateContext;
use Shopware\Core\Framework\Plugin\Context\InstallContext;
use Shopware\Core\Framework\Plugin\Context\UninstallContext;
use Shopware\Core\Framework\Plugin\Util\PluginIdProvider;

class ConnectPayPayment extends Plugin
{
    public const PAYMENT_HANDLER_IDENTIFIER = 'connect_pay.shopware_payment.payment_handler.async';

    /**
     * Runs when plugin is activated
     *
     * @param ActivateContext $activateContext
     */
    public function activate(ActivateContext $activateContext): void
    {
        $this->paymentsInstaller($activateContext)->activate();

        parent::activate($activateContext);
    }

    /**
     * Runs when plugin is deactivated
     *
     * @param DeactivateContext $deactivateContext
     */
    public function deactivate(DeactivateContext $deactivateContext): void
    {
        $this->paymentsInstaller($deactivateContext)->deactivate();

        parent::deactivate($deactivateContext);
    }

    /**
     * Runs when plugin is installed
     *
     * @param InstallContext $installContext
     */
    public function install(InstallContext $installContext): void
    {
        $this->paymentsInstaller($installContext)->install();

        parent::install($installContext);
    }

    /**
     * Runs when plugin is uninstalled
     *
     * @param UninstallContext $uninstallContext
     */
    public function uninstall(UninstallContext $uninstallContext): void
    {
        $this->paymentsInstaller($uninstallContext)->uninstall();

        parent::uninstall($uninstallContext);
    }

    /**
     * Initialize payments installer
     *
     * @param InstallContext $context
     * @return PaymentInstaller
     */
    private function paymentsInstaller(InstallContext $context): PaymentInstaller
    {
        return new PaymentInstaller(
            $context->getContext(),
            $this->container->get(PluginIdProvider::class),
            $this->container->get('payment_method.repository'),
            $this->container->get('language.repository'),
            $this->container->get('media.repository'),
            $this->container->get(FileSaver::class)
        );
    }
}
