import template from './providers-settings.html.twig';

const { Criteria } = Shopware.Data;
const { Component } = Shopware;

Component.register('providers-settings', {
    template,
    
    inject: ['repositoryFactory', 'systemConfigApiService'],
    
    data() {
        return {
            country: 0,
            providers: {
                type: Object,
                required: true
            },
            Countries: null
        };
    },
    
    props: {
        criteria: {
            type: Object,
            required: false,
            default() {
                return new Criteria();
            },
        },
    },
    
    computed: {
        CountriesRepository() {
            return this.repositoryFactory.create('cp_providers_countries');
        },
        ProvidersRepository() {
            return this.repositoryFactory.create('cp_bank_providers');
        },
        PluginConfig() {
            return this.systemConfigApiService.getValues('ConnectPayPayment.config').then((data) => {
                if (data['ConnectPayPayment.config.providersSettingsCountry']) {
                    this.country = data['ConnectPayPayment.config.providersSettingsCountry'];
                    this.updateProviders(this.country, false);
                }
                if (data['ConnectPayPayment.config.providersSettingsProvider']) {
                    this.multiValue = data['ConnectPayPayment.config.providersSettingsProvider'].split('|');
                }
            });
        },
        context() {
            return { ...Shopware.Context.api, inheritance: true };
        },
        
        multiValue: {
            get() {
                if (this.country === null || this.country === 0) {
                    return [];
                }

                if (typeof this.providers === 'string' && this.providers) {
                    return this.providers.split('|');
                }
            },
            set(values) {
                this.providers = values.join('|');
            }
        }
    },
    
    created() {
        this.PluginConfig;
        this.loadData();
    },
    
    methods: {
        
        loadData() {
            
            const criteria = new Criteria();
            
            this.CountriesRepository.search(
                criteria,
                Shopware.Context.api
            ).then((response) => {
                this.Countries = response;
                return response;
            });
        },
        
        updateProviders(country = 0, resetProviders = false) {
            
            this.systemConfigApiService.saveValues({'ConnectPayPayment.config.providersSettingsCountry': country});
            
            if (resetProviders) {
                this.updateProvider();
            }

            this.criteria = new Criteria();
            this.criteria.addFilter(
                Criteria.equals('countryCode', this.country)
            );
        },
        
        updateProvider(providers = '') {
            if (providers === '') {
                this.multiValue = [];
            }
        },
        
        updateProvidersData() {
            var data = '';
            this.loading = true;
            if (this.multiValue) {
                data = this.multiValue.join('|');
            }
            this.systemConfigApiService.saveValues({'ConnectPayPayment.config.providersSettingsProvider': data});
            location.reload();
        }
    }
});
